/* gen_menu - generate C source for menu from description file */
#include "local.h"
#define MAXBFLDS 8000
#define C_ID_LEN 31
#define FILE_NM_LEN 64		/* max length of file name */
#define C_EXPR_LEN 80		/* arbitrarly limit on length of struct ref */
#define TITLE_LEN 80		/* max length of displayed title */
#define SCR_POS_LEN 3		/* max digits in screen position */
#define FTYPE_LEN 1			/* field type is only one char */
#define EDIT_LEN 2			/* edit type is 1 char plus 1 optional digit */
#define MAXFLINE (TITLE_LEN+FTYPE_LEN+EDIT_LEN+SCR_POS_LEN+SCR_POS_LEN+50)
#define GETPSTR(p, s)  \
	{  if (!getpstr(p, s, sizeof(s)))  error("Cannot match" , p);  }
#define OPTPSTR(p, s)  \
	getpstr(p, s, sizeof(s))
#define GETPLIN(p, s)  \
	{  if (!getplin(p, s, sizeof(s)))  error("Cannot match" , p);  }																					                                                                         /*
.PE
.PS 17
/* internal variables */
static char buf[BUFSIZ] = {""};
static char buf_flds[MAXBFLDS] = {""};
static short i_flds = 0;
static char mname[C_ID_LEN+1] = {""};
static char mstruct[C_EXPR_LEN+1] = {""};
static char mtitle[TITLE_LEN+1] = {""};
static char mheader[FILE_NM_LEN+1] = {""};
static char fname[C_ID_LEN+1] = {""};
static char fline[SCR_POS_LEN+1] = {""};
static char fcol[SCR_POS_LEN+1] = {""};
static char ffn[C_ID_LEN+1] = {""};
static char ftype[FTYPE_LEN+1] = {""};
static char fedit[EDIT_LEN+1] = {""};
static char fdesc[TITLE_LEN+1] = {""};
void do_1_menu(), do_1_fld(), pr_s_fld(), pr_c_fld(), pr_m_fld(), pr_f_fld();																					                                                                         /*
.PE
.PS 15
/* gen_menu (main) */
void main()
	{
	do_1_menu();
	while (OPTPSTR("Field name:", fname))
		do_1_fld();		/* process one field */
	printf("static FIELD F%s[] =\n", mname);
	printf("\t{\n");
	for (i_flds = 0; buf_flds[i_flds] != '\0'; ++i_flds)
		putchar(buf_flds[i_flds]);
	printf("\t};\n");
	printf("MENU %s = {\"%s\", F%s, DIM(F%s), 0};\n",
		mname, mtitle, mname, mname);
	}																					                                                                         /*
.PE
.PS 13
/* do_1_menu - process the menu information */
void do_1_menu()
	{
	GETPSTR("Menu name:", mname);
	GETPLIN("Menu title:", mtitle);
	GETPSTR("Header:", mheader);
	GETPSTR("C-struct:", mstruct);
	printf("/* %s - %s */\n", mname, mtitle);
	printf("#include \"local.h\"\n");
	printf("#include \"menu.h\"\n");
	printf("#include \"%s\"\n", mheader);
	}																					                                                                         /*
.PE
.PS 33
/* do_1_fld - process the info for one field */
void do_1_fld()
	{
	asserts(IN_RANGE(i_flds, 0, MAXBFLDS - MAXFLINE), "buffer space ok"); 
	GETPLIN("Desc:", fdesc);
	GETPSTR("Line:", fline);
	GETPSTR("Col:", fcol);
	GETPSTR("Type:", ftype);
	if (tolower(ftype[0]) == 's')
		GETPSTR("Edit:", fedit);
	if (ftype[0] != 'm')
		GETPSTR("Post-fn:", ffn);
	if (!STREQ(ffn, "0") && !STREQ(ffn, "NULL"))
		printf("extern bool %s();\n", ffn);
	switch (ftype[0])
		{
	case 's':
	case 'S':
		pr_s_fld();
		break;
	case 'c':
		pr_c_fld();
		break;
	case 'm':
		pr_m_fld();
		break;
	case 'f':
		pr_f_fld();
		break;
	default:
		error("Unknown field type", ftype);
		}
	i_flds += strlen(&buf_flds[i_flds]);
	if (i_flds > MAXBFLDS - MAXFLINE)
		error("out of buffer space", "");
	}																					                                                                         /*
.PE
.PS 22
/* pr_s_fld - print the info for a string field */
void pr_s_fld()
	{
	sprintf(&buf_flds[i_flds],
		"\t{%s, %s, \"%s\", '%c', &S%s, NULL, NULL, %s},\n",
		fline, fcol, fdesc, ftype[0], fname, ffn);
	if (fedit[1] == '\0')
		{
		printf("static STRING_VAL S%s =\n", fname);
		printf("\t{'%c', (%s).%s, sizeof((%s).%s), NULL};\n",
			fedit[0], mstruct, fname, mstruct, fname);
		}
	else
		{
		printf("static char N%s[%c+1] = {0};\n",
			fname, fedit[1]);
		printf("static STRING_VAL S%s =\n", fname);
		printf("\t{'%c', N%s, sizeof(N%s), &(%s).%s};\n",
			fedit[0], fname, fname, mstruct, fname);
		}
	}																					                                                                         /*
.PE
.PS 18
/* pr_c_fld - print the info for a choice field */
void pr_c_fld()
	{
	int ret;

	sprintf(&buf_flds[i_flds],
		"\t{%s, %s, \"%s\", '%c', NULL, &C%s, NULL, %s},\n",
		fline, fcol, fdesc, ftype[0], fname, ffn);
	printf("static char *T%s[]=\n", fname);
	do	{
		ret = getsnn(buf, BUFSIZ);
		if (ret != EOF && ret > 0)	/* skip blank lines */
			printf("%s\n", buf);
		} while (ret != EOF && strchr(buf, ';') == NULL);
	printf("static CHOICE_VAL C%s =\n", fname);
	printf("\t{DIM(T%s), T%s, (%s).%s};\n",
		fname, fname, mstruct, fname);
	}																					                                                                         /*
.PE
.PS 15
/* pr_m_fld - print the info for a menu field */
void pr_m_fld()
	{
	printf("extern MENU %s;\n", fname);
	sprintf(&buf_flds[i_flds],
		"\t{%s, %s, \"%s\", '%c', NULL, NULL, &%s, %s},\n",
		fline, fcol, fdesc, ftype[0], fname, ffn);
	}
/* pr_f_fld - print the info for a function field */
void pr_f_fld()
	{
	sprintf(&buf_flds[i_flds],
		"\t{%s, %s, \"%s\", '%c', NULL, NULL, NULL, %s},\n",
		fline, fcol, fdesc, ftype[0], ffn);
	}
